unit FdesTool;

interface
Uses
  DefLib,
  Forms, Dialogs, Classes, SysUtils, Controls, Graphics, WinProcs,
  IniFiles;

const
  C_ProgDescr   = 'File descriptions';
  C_ProgIdn     = 'FDES';
  C_ProgVers    = 'v1.7';
  C_ProgTarget  = {$ifdef WIN32} '(32 bit)' {$else} '(16 bit)' {$endif};
  C_Copyright   = 'Copyright  1997-2000, J.R. Ferguson';
  C_AuthorEmail = 'j.r.ferguson@iname.com';
  C_AuthorURL   = 'http://hello.to/ferguson';
  C_IniFileExt  = '.INI';
  C_LstFileExt  = '.fdc';
  C_LngFileExt  = '.fde';
  C_GrpFileExt  = '.fdg';

  HC_General    = 20;
  HC_MainData   = 30;
  HC_GroupData  = 40;

  C_FileFilter  =
    'DOS name cat (*'+C_LstFileExt+')|*'+C_LstFileExt+'|'+
    'Long name cat (*'+C_LngFileExt+')|*'+C_LngFileExt+'|'+
    'All files (*.*)|*.*';

  C_FnmCol1Dos  = 01; C_FnmCol2Dos  = 10; C_FnmLenDos = 08;
  C_GrpCol1Dos  = 10; C_GrpCol2Dos  = 01; C_GrpLenDos = 08;
  C_DscCol1Dos  = 19; C_DscCol2Dos  = 19; C_DscLenDos = 200;

  C_FnmCol1Lng  = 01; C_FnmCol2Lng  = 10; C_FnmLenLng = 40;
  C_GrpCol1Lng  = 42; C_GrpCol2Lng  = 01; C_GrpLenLng = 08;
  C_DscCol1Lng  = 51; C_DscCol2Lng  = 51; C_DscLenLng = 200;

  C_GrpIdnCol   = 01; C_GrpIdnLen = 08;

                  {....v....1....v....2}
  C_HeaderFnmDos= 'Filename Group    Description';
  C_HeaderGrpDos= 'Group    Filename Description';
                  {....v....1....v....2....v....3....v....4....v....5..}
  C_HeaderFnmLng= 'Filename                                 Group    Description';
  C_HeaderGrpLng= 'Group    Filename                                 Description';
                  {....v....1....v....2}
type
  T_LastFind    = (lf_Undefined,lf_Fnm,lf_Grp,lf_Txt);
  T_CatType     = (ct_DosNames,ct_LongNames);

  T_Options     = record
    DataFile    : String;
    OptCatType  : T_CatType;
    OptGrpOrder,
    OptToolbar,
    OptHeader,
    OptStatusbar,
    OptHints,
    OptSavePos  : boolean;
    WndTop,
    WndLeft,
    WndHeight,
    WndWidth    : integer;
  end;

const
  C_DflOptions  : T_Options =
  ( DataFile    : '';
    OptCatType  : {$ifdef Win32} ct_LongNames {$else} ct_DosNames {$endif};
    OptGrpOrder : false;
    OptToolbar  : true;
    OptHeader   : true;
    OptStatusbar: true;
    OptHints    : true;
    OptSavePos  : true;
    WndTop      : 0;
    WndLeft     : 0;
    WndHeight   : 640;
    WndWidth    : 1000);

var
  GV_ProgIdn    : String[8];
  GV_SettingFnm : TFileName;
  GV_HeaderFnm  : String;
  GV_HeaderGrp  : String;
  GV_FnmCol1, GV_FnmCol2, GV_FnmCol, GV_FnmLen : StpInd;
  GV_GrpCol1, GV_GrpCol2, GV_GrpCol, GV_GrpLen : StpInd;
  GV_DscCol1, GV_DscCol2, GV_DscCol, GV_DscLen : StpInd;

function  ErrMsg(V_Condition: boolean; V_Msg: String): boolean;
function  ToOrdGrp(const V_Item: StpTyp): StpTyp;
function  ToOrdFnm(const V_Item: StpTyp): StpTyp;
function  ToCatType(V_CatType: T_CatType; const V_Item: StpTyp): StpTyp;
procedure SetCatType(V_CatType: T_CatType);
procedure SetGroupOrder(V_GroupOrder: boolean);
function  SetListItem(const V_Fnm, V_Grp, V_Dsc: StpTyp): StpTyp;
function  GetListFnm(const V_Item: StpTyp): StpTyp;
function  GetListGrp(const V_Item: StpTyp): StpTyp;
function  GetListDsc(const V_Item: StpTyp): StpTyp;
function  ReadOptions(var V_Options: T_Options): boolean;
procedure WriteOptions(var V_Options: T_Options);


implementation
Uses
  FdesMain, FdesGLst, FdesEadd, FdesEchg, FdesSfnm,
  StfLib, StpLib, NumLib;

function ErrMsg(V_Condition: boolean; V_Msg: String): boolean;
begin
  if V_Condition then
    MessageDlg(GV_ProgIdn+': '+V_Msg,mtError,[mbOK],0);
  Result:= V_Condition;
end;

function  ToOrdGrp(const V_Item: StpTyp): StpTyp;
begin
  Result:= StfSub(V_Item,GV_GrpCol1,GV_GrpLen)+' '+
           StfSub(V_Item,GV_FnmCol1,GV_FnmLen)+' '+
           StfSub(V_Item,GV_DscCol1,GV_DscLen);
end;

function  ToOrdFnm(const V_Item: StpTyp): StpTyp;
begin
  Result:= StfSub(V_Item,GV_FnmCol2,GV_FnmLen)+' '+
           StfSub(V_Item,GV_GrpCol2,GV_GrpLen)+' '+
           StfSub(V_Item,GV_DscCol2,GV_DscLen);
end;

function  ToCatType(V_CatType: T_CatType; const V_Item: StpTyp): StpTyp;
begin case V_CatType of
  ct_DosNames  : ToCatType:= StfUpp(StfSub(V_Item,C_FnmCol1Lng,C_FnmLenDos))
                            +StfSub(V_Item,C_FnmCol1Lng+C_FnmLenLng,255);
  ct_LongNames : ToCatType:= StfIns(V_Item,
                                    StfMake(' ',C_FnmLenLng - C_FnmLenDos),
                                    C_FnmCol1Dos + C_FnmLenDos);
end; end;

procedure SetCatType(V_CatType: T_CatType);
begin
  case V_CatType of
    ct_DosNames : begin
      GV_FnmCol1:= C_FnmCol1Dos; GV_FnmCol2:= C_FnmCol2Dos; GV_FnmLen:= C_FnmLenDos;
      GV_GrpCol1:= C_GrpCol1Dos; GV_GrpCol2:= C_GrpCol2Dos; GV_GrpLen:= C_GrpLenDos;
      GV_DscCol1:= C_DscCol1Dos; GV_DscCol2:= C_DscCol2Dos; GV_DscLen:= C_DscLenDos;
      GV_HeaderFnm:= C_HeaderFnmDos; GV_HeaderGrp:= C_HeaderGrpDos;
      with MainWnd do begin
        DlgFileOpen.DefaultExt := StfAfter(C_LstFileExt,'.');
        DlgFileSave.DefaultExt := StfAfter(C_LstFileExt,'.');
        DlgFileOpen.FilterIndex:= 1;
        DlgFileSave.FilterIndex:= 1;
      end;
    end;
    ct_LongNames : begin
      GV_FnmCol1:= C_FnmCol1Lng; GV_FnmCol2:= C_FnmCol2Lng; GV_FnmLen := C_FnmLenLng;
      GV_GrpCol1:= C_GrpCol1Lng; GV_GrpCol2:= C_GrpCol2Lng; GV_GrpLen := C_GrpLenLng;
      GV_DscCol1:= C_DscCol1Lng; GV_DscCol2:= C_DscCol2Lng; GV_DscLen := C_DscLenLng;
      GV_HeaderFnm:= C_HeaderFnmLng; GV_HeaderGrp:= C_HeaderGrpLng;
      with MainWnd do begin
        DlgFileOpen.DefaultExt := StfAfter(C_LngFileExt,'.');
        DlgFileSave.DefaultExt := StfAfter(C_LngFileExt,'.');
        DlgFileOpen.FilterIndex:= 2;
        DlgFileSave.FilterIndex:= 2;
      end;
    end;
  end;
  MainWnd.Options.OptCatType:= V_CatType;
  SetGroupOrder(MainWnd.Options.OptGrpOrder);
end;

procedure SetGroupOrder(V_GroupOrder: boolean);
begin
  MainWnd.Options.OptGrpOrder := V_GroupOrder;
  if V_GroupOrder then begin
    GV_FnmCol:= GV_FnmCol2;
    GV_GrpCol:= GV_GrpCol2;
    GV_DscCol:= GV_DscCol2;
    MainWnd.Header.Caption:= GV_HeaderGrp;
  end
  else begin
    GV_FnmCol:= GV_FnmCol1;
    GV_GrpCol:= GV_GrpCol1;
    GV_DscCol:= GV_DscCol1;
    MainWnd.Header.Caption:= GV_HeaderFnm;
  end;
end;

function  SetListItem(const V_Fnm, V_Grp, V_Dsc: StpTyp): StpTyp;
begin
  if MainWnd.Options.OptGrpOrder then Result:=
    StfFill(StfTrunc(V_Grp,GV_GrpLen),' ',GV_GrpLen+1)+
    StfFill(StfTrunc(V_Fnm,GV_FnmLen),' ',GV_FnmLen+1)+
            StfRTS  (V_Dsc)
  else Result:=
    StfFill(StfTrunc(V_Fnm,GV_FnmLen),' ',GV_FnmLen+1)+
    StfFill(StfTrunc(V_Grp,GV_GrpLen),' ',GV_GrpLen+1)+
            StfRTS  (V_Dsc);
end;

function  GetListFnm(const V_Item: StpTyp): StpTyp;
begin
  if MainWnd.Options.OptCatType = ct_Dosnames then
    Result:= StfUpp(StfRTS(StfSub(V_Item,GV_FnmCol,GV_FnmLen)))
  else
    Result:= StfRTS(StfSub(V_Item,GV_FnmCol,GV_FnmLen));
end;

function  GetListGrp(const V_Item: StpTyp): StpTyp;
begin Result:= StfRTS(StfSub(V_Item,GV_GrpCol,GV_GrpLen)); end;

function  GetListDsc(const V_Item: StpTyp): StpTyp;
begin Result:= StfRTS(StfSub(V_Item,GV_DscCol,GV_DscLen)); end;

function ReadOptions(var V_Options: T_Options): boolean;
var IniFile: TIniFile; ok: boolean;

  procedure GetFont(const V_Section: String; V_Font: TFont);
  var bold, italic: boolean;
  begin with IniFile, V_Font do begin
    Name   := ReadString (V_Section, 'Name'  ,Name  );
    Size   := ReadInteger(V_Section, 'Size'  ,Size  );
    Color  := ReadInteger(V_Section, 'Color' ,Size  );
    bold   := ReadBool   (V_Section, 'Bold'  ,fsBold   in Style);
    italic := ReadBool   (V_Section, 'Italic',fsItalic in Style);
    Style:= [];
    if bold   then Style:= Style + [fsBold];
    if italic then Style:= Style + [fsItalic];
  end end;

begin with V_Options do begin {ReadOptions}
  ok:= false;
  Screen.Cursor:= crHourGlass;
  IniFile:= TIniFile.Create(GV_SettingFnm);
  try
    with IniFile do begin
      DataFile    := ReadString ('Data'    ,'FileName'  ,DataFile     );
      OptCatType  :=
           T_CatType(ReadInteger('Data'    ,'CatType'   ,ord(ct_DosNames)));
      OptGrpOrder := ReadBool   ('Options' ,'ByGroup'   ,OptGrpOrder  );
      OptToolbar  := ReadBool   ('Options' ,'Toolbar'   ,OptToolbar   );
      OptHeader   := ReadBool   ('Options' ,'Header'    ,OptHeader    );
      OptStatusBar:= ReadBool   ('Options' ,'Statusbar' ,OptStatusBar );
      OptHints    := ReadBool   ('Options' ,'Hints'     ,OptHints     );
      OptSavePos  := ReadBool   ('Options' ,'SavePos'   ,OptSavePos   );
      WndTop      := ReadInteger('Position','Top'       ,WndTop       );
      WndLeft     := ReadInteger('Position','Left'      ,WndLeft      );
      WndHeight   := ReadInteger('Position','Height'    ,WndHeight    );
      WndWidth    := ReadInteger('Position','Width'     ,WndWidth     );
    end;
    with MainWnd do begin
      GetFont('ListFont'   ,ListBox.Font);
      GetFont('GrpListFont',GrpLstFont);
      Header.Font:= ListBox.Font;
    end;
    ok:= true;
  finally
    IniFile.Free;
    Screen.Cursor:= crDefault;
    Result:= ok;
  end;
end; end;

procedure WriteOptions(var V_Options: T_Options);
var IniFile: TIniFile;

  procedure PutFont(const V_Section: String; V_Font: TFont);
  begin with IniFile, V_Font do begin
    WriteString (V_Section,'Name'  ,Name             );
    WriteInteger(V_Section,'Size'  ,Size             );
    WriteInteger(V_Section,'Color' ,Color            );
    WriteBool   (V_Section,'Bold'  ,fsBold   in Style);
    WriteBool   (V_Section,'Italic',fsItalic in Style);
  end end;

begin with V_Options do begin {WriteOptions}
  IniFile:= TIniFile.Create(GV_SettingFnm);
  Screen.Cursor:= crHourGlass;
  try try
    if OptSavePos then with MainWnd do begin
      WndTop    := Top;
      WndLeft   := Left;
      WndHeight := Height;
      WndWidth  := Width;
    end;
    with IniFile do begin
      WriteString ('Data'       ,'FileName'  ,DataFile     );
      WriteInteger('Data'       ,'CatType'   ,ord(OptCatType));
      WriteBool   ('Options'    ,'ByGroup'   ,OptGrpOrder  );
      WriteBool   ('Options'    ,'Header'    ,OptHeader    );
      WriteBool   ('Options'    ,'Toolbar'   ,OptToolbar   );
      WriteBool   ('Options'    ,'Statusbar' ,OptStatusBar );
      WriteBool   ('Options'    ,'Hints'     ,OptHints     );
      WriteBool   ('Options'    ,'SavePos'   ,OptSavePos   );
      WriteInteger('Position'   ,'Top'       ,WndTop       );
      WriteInteger('Position'   ,'Left'      ,WndLeft      );
      WriteInteger('Position'   ,'Height'    ,WndHeight    );
      WriteInteger('Position'   ,'Width'     ,WndWidth     );
      PutFont     ('ListFont'   ,MainWnd.ListBox.Font);
      PutFont     ('GrpListFont',DlgGrpLst.GrpList.Font);
    end;
  finally Screen.Cursor:= crDefault; IniFile.Free; end;
  except on E: Exception do MessageDlg(E.Message,mtError,[mbOK],0) end;
end; end;

end.
